from pathlib import Path
import bpy
from bpy.types import Operator

from ..definitions import ALLEY_ASSET_COLLECTION, ALLEY_MAIN_COLLECTION, ROOT_PACKAGE
from .mixins import Procedural_Alleys_Append_Mixin, exclude_collection
class Procedural_Alleys_OT_Add_Alley(Operator, Procedural_Alleys_Append_Mixin):
    bl_idname = "procedural_alleys.add_alley"
    bl_label = "Add Alley"
    bl_description = "Add Alley of Procedural Alleys"
    bl_options = {"REGISTER","UNDO"}

    @classmethod
    def poll(cls, context):
        props = context.scene.procedural_alleys
        if context.mode != 'OBJECT':
            cls.poll_message_set('You should be in Object mode!')
            return False
        return props.alley_category not in (None,'')
    
    def get_path_to_alley_blendfile(self,context):
        props = context.scene.procedural_alleys
        asset_dir = context.preferences.addons[ROOT_PACKAGE].preferences.filepath
        alleys_path = Path(asset_dir)/props.alley_category
        path_to_blend = next(alleys_path.glob("*.blend"))
        return path_to_blend
    
    def execute(self, context):
        props = context.scene.procedural_alleys
        path_to_blend = self.get_path_to_alley_blendfile(context)

        with bpy.data.libraries.load(str(path_to_blend), link=False) as (data_from, data_to):
            data_to.collections = [coll for coll in data_from.collections if coll in (ALLEY_MAIN_COLLECTION,ALLEY_ASSET_COLLECTION)]

        wrapper_coll = self.ensure_and_get_collection(f'Alley - {props.alley_category}')
        for collection in data_to.collections:
            if ALLEY_MAIN_COLLECTION in collection.name:
                collection.name = props.alley_category
            wrapper_coll.children.link(collection)
        
        for collection in data_to.collections:
            if ALLEY_ASSET_COLLECTION in collection.name:
                exclude_collection(context,collection.name,exclude=True)
        return {'FINISHED'}

classes = [Procedural_Alleys_OT_Add_Alley]
