from dataclasses import dataclass
import bpy
from bpy.types import PropertyGroup, Scene
from bpy.props import EnumProperty, PointerProperty, BoolProperty
from pathlib import Path
from .t3dn_bip import previews


_item_map = dict()

def _make_item(enum_name, id_, name, descr, preview_id, uid):
    lookup = f"{str(id_)}\0{str(name)}\0{str(descr)}\0{str(preview_id)}\0{str(uid)}"
    if enum_name not in _item_map:
        _item_map[enum_name] = {}
    if lookup not in _item_map[enum_name]:
        _item_map[enum_name][lookup] = (id_, name, descr, preview_id, uid)
    return _item_map[enum_name][lookup]

class Procedural_Alleys_Properties(PropertyGroup):
    @staticmethod
    def get_thumbnail(dir:Path, default=0):
        global thumbnails
        # `thumbnail_path` is a variable that stores the path to the thumbnail image file for a
        # specific directory. It is obtained by using the `glob` method from the `pathlib` module to
        # search for a file named "*.png" in the given directory. The `next` function is then
        # used to retrieve the first matching file path found. If no thumbnail file is found, the
        # variable will be set to `None`.
        thumbnail_path = next(dir.glob("*.png"), None)
        thumbnail = default
        if thumbnail_path:
            thumbnail = thumbnails.load_safe(dir.name.lower(),str(thumbnail_path),'IMAGE').icon_id

        return thumbnail

    def get_alley_categories(self, context):
        enum_items = []
        asset_dir = context.preferences.addons[__package__].preferences.filepath
        if not asset_dir:
            return enum_items

        path = Path(asset_dir)

        for i, dir in enumerate(path.iterdir()):
            if not dir.is_dir():
                continue

            thumbnail = self.get_thumbnail(dir, default='SYSTEM')
            enum_items.append(_make_item('alley_category', dir.name, dir.name, '', thumbnail, i))
        return enum_items

    alley_category: bpy.props.EnumProperty(
        items=get_alley_categories, 
        name='Category', 
        description='Category of Alley', 
        default=2
        )

classes = [Procedural_Alleys_Properties]

def register():
    global thumbnails
    thumbnails = previews.new(max_size=(128, 128))
    for cls in classes:
        bpy.utils.register_class(cls)
    Scene.procedural_alleys = PointerProperty(type=Procedural_Alleys_Properties)

def unregister():
    global thumbnails
    del thumbnails
    del Scene.procedural_alleys
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)