import bpy
from bpy.types import Panel, UILayout
from .draw_geonode_ui import nodetree_inputs,draw_gn_mod_props,get_gn_mod, ALLEYS_DRAW_GEONODE_UI_OT_property_toggle
from .definitions import MODIFIER_NAME

import os
import json
import webbrowser

class Procedural_Alleys_PT_GeoNode(Panel):
    bl_idname = "PROCEDURAL_ALLEYS_PT_GeometryNodes"
    bl_label = "Alleys Settings"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Procedural Alleys'
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
        layout: UILayout = self.layout
        if not context.active_object:
            box = layout.box()
            box.label(text="No Active Object", icon='INFO')
            return
        
        mod = get_gn_mod(context.active_object, MODIFIER_NAME)
        if not mod:
            box = layout.box()
            box.label(text="No Procedural Alley Found.", icon='INFO')
            return

        col = layout.column(align=True)
        _layout = col
        for input in nodetree_inputs(mod.node_group):
            if input.identifier == "Input_0": #skip the first input which is always Geoemetry Input
                continue
            _layout = draw_gn_mod_props(_layout,col,mod,input)


class Procedural_Alleys_PT_Categories(Panel):
    bl_idname = "PROCEDURAL_ALLEYS_PT_Categories"
    bl_label = "Procedural Alleys"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Procedural Alleys'

    def draw(self, context):
        layout = self.layout
        props = context.scene.procedural_alleys

        alleys_dir = context.preferences.addons[__package__].preferences.filepath
        if not alleys_dir:
            box=layout.box()
            box.label(text="Select Asset Folder in Preferences", icon='ERROR')
            box.operator('procedural_alleys.open_preferences')
            return
        
        row = layout.row()
        row.template_icon_view(props,"alley_category", show_labels=True)
        
        row = layout.row()
        row.alignment = 'CENTER'
        row.label(text=props.alley_category)
        row = layout.row()
        row.operator('procedural_alleys.add_alley')




class Procedural_Alleys_PT_Help(Panel):
    bl_idname = "Procedural_Alleys_PT_Help"
    bl_label = "Help"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Procedural Alleys'

    @classmethod
    def poll(cls, context):
        return True

    def draw(self, context):
        dns = bpy.app.driver_namespace
        links = dns.get("alleys_links")
        if not links:
            get_links()
        layout = self.layout
        
        if links:
            for link in links:
                label = link["name"]
                url = link["link"]
                icon = link["icon"]
                row = layout.row()
                row.operator("procedural_alleys.url_open", text=label, icon=icon).url = url



class Procedural_Alleys_OT_OpenURL(bpy.types.Operator):
    bl_idname = "procedural_alleys.url_open"
    bl_label = "Open URL"

    url: bpy.props.StringProperty()

    def execute(self, context):
        webbrowser.open(self.url)
        return {'FINISHED'}



def get_links():
    dns = bpy.app.driver_namespace
    dns["alleys_links"] = {}
    current_folder = os.path.dirname(os.path.realpath(__file__))
    links_json = os.path.join(current_folder, "links.json")
    if os.path.exists(links_json):
        with open(links_json, "r") as f:
            dns["alleys_links"] = json.load(f)



classes = [
    Procedural_Alleys_PT_Categories,
    Procedural_Alleys_PT_GeoNode,
    ALLEYS_DRAW_GEONODE_UI_OT_property_toggle,
    Procedural_Alleys_PT_Help,
    Procedural_Alleys_OT_OpenURL,
]

def register():
    for c in classes:
        bpy.utils.register_class(c)


def unregister():
    for c in reversed(classes):
        bpy.utils.unregister_class(c)
